/* ====================================================================
 * 
 *  itrslt.h  -- header file for package itrslt
 *
 *  Routines to be called from Second Level Trigger Processes
 *  to sparsify ITR data
 *  and find hits in Regions of Interest (not implemented yet)
 *
 *  see itrslt.c for more comments on functionality
 *
 *  Author:  (based on vdsslt)
 *           A. Lange
 *           W. Gradl
 *
 *  $Id: itrslt.h,v 1.1.1.7 2002/02/07 16:29:20 gradl Exp $
 *
 * ====================================================================*/

#ifndef __ITRSLT_H__
#define __ITRSLT_H__

/* include these headers just in case ... */
#include "event.h"
#include "etb.h"
#include "esp.h"
#include "grr.h"
#include "cif.h"


#if defined(__cplusplus)
extern "C" {
#endif


#include "iol.h"
typedef struct {
  IOL_MNAME  module[IOL_MLEN];        /* Module name must be first       */
  IOL_SIZE   size;                    /* # of verbs must be second       */ 
  IOL_STRING version;                 /* version of library              */
  IOL_INT    debug_mode;              /* Flag for debugging              */
  IOL_INT    CNA;                     /* 1: use CNA manager              */
} Itrslt_Iol_t;                                     
extern Itrslt_Iol_t itrslt_iol;




/* --------------------------------------------------------------------
     interface functions to be called by CIF
   -------------------------------------------------------------------- */
int CIF_ITR_init(void);
int CIF_ITR_sparse_event(SLB_HDR ** slbs, int n_slbs, void *dest,
			 int maxsize, ESP_machine into_fmt);

int CIF_ITR_sparse_roi(void *source, void *dest, int maxsize, 
		       ESP_machine into_fmt, SLB_SUBHDR *fed,
		       int first_channel, int size, int *nChan);


/* --------------------------------------------------------------------
    local function definitions 
    these should not be called from outside!
   -------------------------------------------------------------------- */
void add_channels(int *pflag, int n);
int ITR_chip_sparsedata(ITR_Marvin_Chip_t * chip);
ITR_Silraw_Event_t *ITR_silraw_get_CIF(SLB_HDR ** slb, int nslb, int ext_format);
int ITR_sparse_event(SLB_HDR ** slbs, int n_slbs, int into_fmt);
char pack(float f);
int pack_subslb_data(int cid, void *dest, ESP_machine into_fmt);



#if defined(__cplusplus)
	   }
#endif


#endif /* __ITRSLT_H__ */
