#!/bin/bash
#
# written by Thorsten Glebe, Jun. 1999
# Max-Planck-Institut of Nuclear Physics
# Heidelberg / Germany
#
# 1. argument: tar-file destination directory
# 2. argument: html-file destination directory
#
# This script creates the WWW homepage of the ITRlib
# project. Location: http://www.mpi-hd.mpg.de/herab/itrlib
#

DIR=`dirname $0`
source $DIR/util.sh

if [ $# -ne 2 ]; then
    echo "usage: $PROG <tarfile-directory> <html-directory>"
    exit 1
else
    DIR=$1
    if [ ! -d "$DIR" ]; then
      error "Directory $DIR not existing!"
    fi
    HTMLDIR=$2
    if [ ! -d "$HTMLDIR" ]; then
      error "Directory $HTMLDIR not existing!"
    fi
fi

#
# Setup xcvs
#
set_xcvs
if [ "$CVSROOT" != "/home/hbitr/CVS" ]; then
  error "CVSROOT ($CVSROOT) not set correctly, must be: /home/hbitr/CVS"
fi

PACK_FILE=$LIBROOT/PACKAGES

#
# extract ITRlib version
#
if [ -f "./CVStag.itrlib" ]; then
  VERSION=`${AWK} '/xcvsVERSION_itrlib/ {print $3}' ./CVStag.vdslib | tr -d \" | cut -c2- | tr / .`
else
  error "No CVStag.itrlib found!!"
fi

echo "Create HTML Documentation page $HTMLDIR/index.html.[en,de]"
#
# Create Page header
#
(
${LIBROOT}/doc/htmlheader.sh $LIBROOT "The HERA-B ITRlib Homepage"

# current ITRlib
echo "<h1><center>Current ITRlib is: <blink>${VERSION}</blink> </center></h1>"

# Package versions
echo "<h2> Packages contained in ITRlib $VERSION: </h2>"
echo "<TABLE cellpadding=\"2\">"
echo "<TR><TH> Package Name <TH> Version Number <TH> Changelog <TH> Documentation <TH> Previous Version <TH> Status"
for i in `cat $PACK_FILE | ${AWK} '!/^[ ]*#.*$/ && $2 ~ /\/[0-9]+-[0-9]+/ {gsub("/",".",$2); print $1"+"substr($2,2)}'`; do
    NAME=`echo $i | cut -d+ -f1`
    VERS=`echo $i | cut -d+ -f2`
    PKGNAME=packages/$NAME/$NAME$VERS/src
    #
    # Repository log info
    #
    if [ -f "${HTMLDIR}/${NAME}.log" ]; then
      echo "remove ${HTMLDIR}/${NAME}.log" >&2
      rm ${HTMLDIR}/${NAME}.log
    fi
    $XCVS log $NAME $VERS > ${HTMLDIR}/${NAME}.log

    #
    # Contents
    #
    if [ -f "${HTMLDIR}/${NAME}_cnt.html" ]; then
      echo "remove ${HTMLDIR}/${NAME}_cnt.html" >&2
      rm ${HTMLDIR}/${NAME}_cnt.html
    fi
    ${LIBROOT}/doc/htmlheader.sh $LIBROOT "$NAME Contents" > ${HTMLDIR}/${NAME}_cnt.html
    echo "<h2> $NAME Contents: </h2>" >> ${HTMLDIR}/${NAME}_cnt.html
    for i in `$XCVS list $NAME $VERS`; do
      echo "<img align=top src=\"/herab/images/greenball.gif\"> $i <br>" >> ${HTMLDIR}/${NAME}_cnt.html
    done
    ${LIBROOT}/doc/htmltrailer.sh $LIBROOT "T.Glebe@mpi-hd.mpg.de" >> ${HTMLDIR}/${NAME}_cnt.html
    chmod a+x ${HTMLDIR}/${NAME}_cnt.html
      
    #
    # Package README's
    #
    DOCFILE="${ITRROOT}/${PKGNAME}/README"
    if [ -f "${DOCFILE}" ]; then
      LINKNAME=${NAME}_`basename $DOCFILE`
      if [ -L ${HTMLDIR}/${LINKNAME} ]; then
        echo "remove ${HTMLDIR}/${LINKNAME}" >&2
	rm ${HTMLDIR}/${LINKNAME}
      fi
      echo "link ${HTMLDIR}/${LINKNAME} -> $DOCFILE" >&2
      ln -s $DOCFILE ${HTMLDIR}/${LINKNAME}
      DOCLINK="<a href=\"$LINKNAME\"> `basename $DOCFILE` </a>"
    else
      DOCLINK=""
    fi

    # Previous version
    OLD_VERS=`${AWK} 'BEGIN{FS="//"} /VersionMSPRIV/ {gsub("VersionMSPRIV","//"); print $2}' ${ITRROOT}/${PKGNAME}/CVStag.$NAME`
    # Status
    STATUS=`${AWK} '/^\/\*TIME\/-[0-9]+/ {print "development"}' ${ITRROOT}/${PKGNAME}/CVStag.$NAME`

    echo "<TR><TD align=right><a href=\"${NAME}_cnt.html\"> $NAME </a> <TD align=center> $VERS <TD align=left> <a href=\"${NAME}.log\"> $NAME </a> <TD align=center> $DOCLINK <TD align=center> $OLD_VERS <TD align=center> ${STATUS:-stable}"
done
echo "</TABLE>"

#
# general documentation
#
echo "<h2>General documentation</h2>"
if [ -L "${HTMLDIR}/README.ITR" ]; then
  rm -f ${HTMLDIR}/README.ITR
fi
ln -s ${ITRROOT}/libitr${VERSION}/README ${HTMLDIR}/README.VDS
echo "<img align=top src=\"/herab/images/greenball.gif\"> <a href=\"README.ITR\"> VDSlib Readme </a><br>"
if [ -f "${HTMLDIR}/itrlib.log" ]; then
    echo "remove ${HTMLDIR}/itrlib.log" >&2
    rm ${HTMLDIR}/itrlib.log
fi
$XCVS log itrlib $VERSION > ${HTMLDIR}/vdslib.log
echo "<img align=top src=\"/herab/images/greenball.gif\"> <a href=\"itrlib.log\"> ITRlib$VERSION Changelog </a><br>"

#
# Further README's
#
for i in `cat $PACK_FILE | ${AWK} '!/^[ ]*#.*$/ && $2 ~ /\/[0-9]+-[0-9]+/ {gsub("/",".",$2); print $1"+"substr($2,2)}'`; do
    NAME=`echo $i | cut -d+ -f1`
    VERS=`echo $i | cut -d+ -f2`
    PKGNAME=packages/$NAME/$NAME$VERS/src
    for i in `ls $ITRROOT/$PKGNAME/README.* 2>/dev/null`; do
      LINKNAME=${NAME}_`basename $i`
      if [ -L ${HTMLDIR}/${LINKNAME} ]; then
        echo "remove ${HTMLDIR}/${LINKNAME}" >&2
	rm ${HTMLDIR}/${LINKNAME}
      fi
      echo "link ${HTMLDIR}/${LINKNAME} -> $i" >&2
      ln -s $i ${HTMLDIR}/${LINKNAME}
      echo "<img align=top src=\"/herab/images/greenball.gif\"> <a href=\"${LINKNAME}\"> $NAME $VERS `basename $i` </a><br>"
    done
done

#
# create tar-file links
#
echo "<h2>ITRlib sources</h2>"
echo "Installation/compilation: (VERS=ITRlib version in question)"
echo "<ol>"
echo "<li> get libitrVERS.src.tar.gz"
echo "<li> cd to \$HBROOT/ITR"
echo "<li> gtar xvzf libitrVERS.src.tar.gz"
echo "<li> cd libitrVERS"
echo "<li> define ITRROOT: export VDSROOT=\$HBROOT/VDS"
echo "<li> to build the library: ./ITRmakelib"
echo "<li> To set pro-links: ./prolink.sh"
echo "<li> send bug reports to: <a href=\"mailto:T.Glebe@mpi-hd.mpg.de\"> T. Glebe </a>"
echo "</ol><p>"
if [ -d "${HTMLDIR}/exp" ]; then
  echo "remove ${HTMLDIR}/exp" >&2
  rm -rf ${HTMLDIR}/exp
fi
mkdir ${HTMLDIR}/exp
for i in ${DIR}/itrlib*; do 
  PNAME=`basename $i`
  TNAME="${HTMLDIR}/exp/$PNAME"
  ln -s $i $TNAME
  echo "<img align=top src=\"/herab/images/greenball.gif\"> <a href=\"/herab/itrlib/exp/$PNAME\"> $PNAME </a><br>"
done

#
# Create Page trailer
#
${LIBROOT}/doc/htmltrailer.sh $LIBROOT "T.Glebe@mpi-hd.mpg.de"
) > $HTMLDIR/index.html.de
chmod a+x $HTMLDIR/index.html.de
cp $HTMLDIR/index.html.de $HTMLDIR/index.html.en

# END
echo "done..."
exit 0
