#!/bin/bash
#
# written by Thorsten Glebe, Jan. 1999
# Max-Planck-Institut of Nuclear Physics
# Heidelberg / Germany
#
# 1. argument: tar-file destination directory
#
AWK=gawk

DIR=`dirname $0`
source $DIR/util.sh

if [ $# -ne 1 ]; then
    echo "usage: $PROG <destination-directory>"
    exit 1
else
    DIR=$1
fi

PACK_FILE=$LIBROOT/PACKAGES

#
# extract ITRlib version
#
if [ -f "./CVStag.itrlib" ]; then
  VERSION=`${AWK} '/xcvsVERSION_itrlib/ {print $3}' ./CVStag.itrlib | tr -d \" | cut -c2- | tr / .`
else
  error "No CVStag.itrlib found!!"
fi

# Name of the .tar file
TARFILE="${DIR}/itrlib${VERSION}.src.tar"

#
# remove potentially existing tar-files in itrlib
#
if [ -f "$TARFILE" ]; then
    echo "removing ${TARFILE}"
    rm $TARFILE
fi
if [ -f "${TARFILE}.gz" ]; then
    echo "removing ${TARFILE}.gz"
    rm ${TARFILE}.gz
fi

#
# tar the itrlib
#
set_tar
$TAR cvf $TARFILE -C ${ITRROOT} libitr${VERSION} --exclude "*/bin/*" --exclude "*/lib/*"
check_status "error tarring ${ITRROOT}/libitr${VERSION}"

#
# collect packages
#
for i in `cat $PACK_FILE | ${AWK} '!/^[ ]*#.*$/ && $2 ~ /\/[0-9]+-[0-9]+/ {gsub("/",".",$2); print $1"+"substr($2,2)}'`; do
    NAME=`echo $i | cut -d+ -f1`
    VERS=`echo $i | cut -d+ -f2`
    PKGNAME=packages/$NAME/$NAME$VERS/src
    if [ -d "$ITRROOT/$PKGNAME" ]; then
       $TAR rvf $TARFILE -C $ITRROOT $PKGNAME 2>>$LIBROOT/ERRORS
       check_status "error tarring $PKGNAME"
    fi
done

#
# gzip the tar-file
#
echo "gzip ${TARFILE}..."
gzip $TARFILE

# END
echo "done..."
exit 0
