#include <stdio.h>
#include <signal.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <ctype.h>
#include <time.h>

#include "aframelib.h"
#include "iol.h"
#include "grr.h"


static AFRAME_STRUCT aframe_struct={
  AFRAME_NAME             ,AFRAME_NVERBS,
  {"version"              ,AFRAME_VERSION   ,        IOL_TYPE_STRING},
  {"debug mode"           ,0                ,  " " , IOL_TYPE_FLAG},
  {"pref path"            ,""               ,        IOL_TYPE_STRING},
  {"preferences"          ,"iol_pref.src"   ,        IOL_TYPE_STRING},
  {"screen log"           ,1                ,  " " , IOL_TYPE_FLAG},
  {"online control mask"  ,GRR_SERROR       ,  " " , IOL_TYPE_HEX}
};

/* program name */
static char aframe_program_name[80] = "noname?";

int aframe_param_init()
{
  iol_declare_structure((IOL_GENERIC *) &aframe_struct);
  return AFRAME_OK;
}


int aframe_process_args (int argc, const char **argv, int mode)
{
  int iarg;
  int iopt;                       /* Option counter in Argument*/
  int endscan;                    /* indicates an arg that uses the full string on cli*/
  int istat;

  /* Process the new arguments */
  for ( iarg=1; iarg<argc; ++iarg ) {
    if ( aframe_struct.debug_mode.value )
      printf("Argument %d is %s\n",iarg,argv[iarg]);
    endscan = 0;
    if ( argv[iarg][0] == AFRAME_OPTCHAR )
      for ( iopt=1; (!endscan)&&(iopt<strlen(argv[iarg])); ++iopt) {
	switch ( argv[iarg][iopt] ) {
	case 'P':
	  istat = aframe_newpref( argv[iarg], iopt+1 );
	  if ( mode == AFRAME_CLI && istat != AFRAME_OK )
	    return istat;
	  endscan = mode == AFRAME_CLI;
	  break;
	default:
	  break;
	}
      }
  }

  return AFRAME_OK;
}


int aframe_newpref( const char *filename, int cptr )
{
  int module_matches;
  int verb_matches;
  char file[250];
  char afm[250];
  char afv[250];

  strcpy(file,filename+cptr);

  /* Load the filename into the parameters */
  strcpy(afm,AFRAME_NAME);
  strcpy(afv,"preferences");
  if(iol_load_value_from_mod_verb(file,afm,afv,
			       &module_matches,&verb_matches)==IOL_OK)

    if ( module_matches && verb_matches )   /* check it was loaded */
      return aframe_load_pref();
    else {
      printf("aframe_newpref: load failed module %s and verb %s \n",
	     module_matches?"matches":"does not match",
	     verb_matches  ?"matches":"does not match"); 
      return AFRAME_NOTOK;
    }
}

int aframe_load_pref()
{
  char filename[200];
  int istat,iolstat;

  aframe_build_filename(filename);  /* builds full name into filename */
  printf("aframe_load_pref: Load %s ",filename);
  istat = (iolstat=iol_scan_preferences(filename))==IOL_OK ? AFRAME_OK: AFRAME_NOTOK;
  printf( istat==AFRAME_OK ? "%s\n": "%s (%d)\n",
	  istat==AFRAME_OK ? "ok": "failed",iolstat);

  return istat;
}

int aframe_store_program_name(const char *pname)
{
  char *name;
  char wholename[1024];  /* wg:  80 is way too small! */

  /* strip pname from directory tree */ 
  strncpy( wholename, pname, 1023 );
  name = strtok( wholename, "/" );
  while ( name ) {
    strncpy( aframe_program_name, name, 80 );
    name = strtok( NULL, "/" );
  }

  return AFRAME_OK;
}

int aframe_output_error_mess(char *mess,char *routine, int errlev)
{
  char mgout[512];

  sprintf( mgout,"%s: %s: %s", aframe_program_name, routine, mess );

  /* print-out warning messages to erwin */
  if ( errlev >= aframe_struct.online_control.value )
    grr_error(errlev,mgout,0);

  /* print to screen */
  if ( aframe_struct.screen_log.value )
    printf( "%s\n", mgout );

  return AFRAME_OK;
}

/* output routine - replaces aframe_output_error_message(...)
 *
 * should be called like printf(...) with the additional 
 * parameters routine and errlev                            */
void aframe_mess(char *routine, int errlev, char *mess, ... )
{
  va_list ap;

  char messtot[512];
  char mgout[512];

  /* handle the variable number of arguments (-> stdarg.h header!) */
  va_start( ap, mess );
  vsprintf( messtot, mess, ap );
  va_end(ap);

  sprintf( mgout,"%s: %s: %s", aframe_program_name, routine, messtot );

  /* print-out warning messages to erwin */
  if ( errlev >= aframe_struct.online_control.value )
    grr_error(errlev,mgout,0);

  /* print to screen */
  if ( aframe_struct.screen_log.value )
    printf( "%s\n", mgout );

}

/* Get the filename.  Try to interpret the path as an environment variable.
   If it does not make sense, try to use the string directly. */
int aframe_build_filename(char *filename)
{
  strcpy(filename,aframe_struct.path.value);
  if ( getenv(filename) == NULL )
    strcpy(filename,aframe_struct.path.value);
  else {
    strcpy(filename,getenv(filename));    
    strcat(filename,"/");
  }
  strcat(filename,aframe_struct.filename.value);

  return AFRAME_OK;
}

