#ifndef SDISP_DEF_H
#define SDISP_DEF_H

/*#define NSPACE          3000000*/                /* size of PAW memory */
#define VERSION         "2.1"
#define DATE            "98/09/24"

#define WAITAFTERDISPLAY 000   /* Time (in msec) to wait after displaying one event */ 

#define PHEIGHT_THRESHOLD 20
#define HELIX_START_1  16
#define HELIX_START_2  150
#define HELIX_CHANNELS 125 

/* Use specific windows */
#define USE_RAWDATA      1
#define USE_WIREMAP      0
#define USE_CLUSTER      0
#define USE_PHEIGHT      1
#define USE_FINFO        0
#define USE_MEANPHEIGHT  0
#define USE_CORRDATA     1
#define USE_FITPARAMS    0

/* Windowtitles */
#define WINTIT_0        "-ICONIFY THIS WINDOW"
#define WINTIT_RAWDATA  "-Rawdata"
#define WINTIT_WIREMAP  "-Wiremap"
#define WINTIT_CLUSTER  "-Clustersize/Clusternumber"
#define WINTIT_PHEIGHT  "-Pulseheight"
#define WINTIT_FINFO    "-File Info"
#define WINTIT_MEAN     "-Mean Pulseheight"
#define WINTIT_CORRDATA "-Corrected Data"
#define WINTIT_PARAMS   "-Fit Parameters"

/* Windows Height and Width */
#define WINH_0              400
#define WINW_0              400
#define WINH_RAWDATA        500
#define WINW_RAWDATA       1000
#define WINH_WIREMAP        500
#define WINW_WIREMAP       1000
#define WINH_CLUSTER        300
#define WINW_CLUSTER        600
#define WINH_PHEIGHT        500
#define WINW_PHEIGHT       1000
#define WINW_FINFO          400  
#define WINH_FINFO          400
#define WINH_MEAN           500
#define WINW_MEAN          1000
#define WINH_CORRDATA       500
#define WINW_CORRDATA      1000
#define WINH_PARAMS         600
#define WINW_PARAMS         600

/* Windows Position in X and Y */ 
#define WINPOSX_0             1
#define WINPOSY_0             1
#define WINPOSX_RAWDATA     100
#define WINPOSY_RAWDATA       1
#define WINPOSX_WIREMAP     400
#define WINPOSY_WIREMAP     100
#define WINPOSX_CLUSTER     400
#define WINPOSY_CLUSTER     200
#define WINPOSX_PHEIGHT     100
#define WINPOSY_PHEIGHT     400
#define WINPOSX_FINFO       500
#define WINPOSY_FINFO       500
#define WINPOSX_MEAN        100
#define WINPOSY_MEAN        100
#define WINPOSX_CORRDATA    120
#define WINPOSY_CORRDATA    120
#define WINPOSX_PARAMS      140
#define WINPOSY_PARAMS      140

/* Histogram identifiers: add an offset 1000*i for PCB # i */
#define HID_RAWDATA     10000
#define HID_WIREMAP     20000
#define HID_CLUSTERSIZE 30000
#define HID_CLUSTERNUM  40000
#define HID_PHEIGHT     50000
#define HID_MEAN        60000
#define HID_CORRDATA    70000
#define HID_BASELINE_INTERCEPT 100000
#define HID_BASELINE_SLOPE     100010

/* Total Number of PCBs to be read out */
#define PCB_NUM             1

/* Number of PCBs per MSGC */
#define PCB_ON_MSCG         3

/* Default settings */
#define DEF_THRESHOLD      30
#define DEF_DISPEVERY       1
#define DEF_CONTMODE        1
#define DEF_RFILENAME       "data.dat"
#define DEF_SFILENAME       "save.dat"
#define DEF_CFILENAME       "sdisp.cfg"
#define DEF_PHEIGHTSTART   0
#define DEF_MAXCLUSTERSIZE 10
#define DEF_MAXCLUSTERNUM  10
#define DEF_MAXPHEIGHT    512
#define DEF_HELIXPCB      0
#define DEF_SHARC         5
#define DEF_LINK          0

#define LIN_CORRECTION      0
#define AVG_CORRECTION      1
#define RUN_AVG_CORRECTION  2
#define FFT_CORRECTION      3

#define AVG_RESTART         0xF0
#define AVG_BASELINE_RUN    0xF1
#define AVG_DATA_RUN        0xF2


/* to be a user changable setting! */
#define MAX_RAWDATA_VALUE 300           /* Set to zero, if not to be used */

#define NAMELENGTH      255
#define STRLENGTH       255
#define FILELINELENGTH  2048
#define EVENTLENGTH     300
#define COMMENTCHAR     '#'
#define HEADERCHARS     "#@"
#define HDRLEN          4

 
typedef struct
{
  int threshold;                  /* Threshold for analysation */ 
  int dispEvery;                  /* Display every ?? events   */
  int contmode;                   /* Mode: read once/cont      */
  char rfilename[NAMELENGTH];     /* Read data from            */
  char sfilename[NAMELENGTH];     /* Save data to              */
  char cfilename[NAMELENGTH];     /* Configuration File Name   */
  int  pheightstart;              /* First channel in pulseheight display  */
  int  ipcb;                      /* Display histograms for this HelixPCB  */
  int  corrmode;                  /* Method used for correcting raw data   */
  int  pcorr;                     /* optional parameter for correction     */
  int  rawmin, rawmax;            /* limits for raw data window            */
  int  corrmin, corrmax;          /* limits for corrdata window            */
} tSettings;

typedef struct
{
  int year;
  int month;
  int day;
  int hour;
  int min;
  int runnumber;
} tHeader;


typedef struct
{
  int evt_time;                   /* unixtime in seconds since 1. 1. 1970 */
  int evt_isharc;                 /* Sharc and link number                */
  int evt_ilink;  
  int evt_ievent;                 /* event number                         */
  int evt_nelements;              /* number of elements                   */
  int evt_event [EVENTLENGTH] ;   /* raw data for this event              */
  int evt_evtt0 [EVENTLENGTH] ;   /* raw data corrected by time jitter    */
  int evt_cdata [EVENTLENGTH] ;   /* corrected data for this event        */
  long numhits;                   /* number of hits found in events       */
} tEvt;


#ifndef min
#define        min(a,b)        ((a) < (b) ? (a) : (b))
#endif

#ifndef max
#define        max(a,b)        ((a) > (b) ? (a) : (b))
#endif

#endif   /* SDISP_DEF_H */
