#include "cif.h"
#include "rpm.h"
#include "rpi.h"
#include <string.h>
#include "grr.h"
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include "itr_cif.h"

static int run = 1;
static int ftab_ids[100];
static int N_ftabs = 0;
static int update_mode = 0;

static int set_mode (int mode,int key){
  printf("set_mode: update mode set to %d for updates key %d\n",mode,key);
  update_mode = mode;
  return GRR_OK;
}


static void ftab_updated(const CIF_formatted_tab *ftab){
  int i;

  if ( !update_mode ){
    printf("ftab_updated: got table update in non update mode ==> will ignore it\n");
    return;
  }
    

  for(i=0;i<N_ftabs;i++)
    if ( ftab->id == ftab_ids[i] ) break;
  if ( i==N_ftabs )
    printf("ftab_updated: id %d not known (know %d ftabs)\n", ftab->id,N_ftabs);
  else
    printf("ftab %d updated for key %d with length %d cont '%s'\n",
	   ftab->id,ftab->key,ftab->len,(char*)(ftab->data));
}

static int handle_rpm(){
  static char rpm_msg_data[RPM_LMAX];
  static rpm_hdr rpm_msg_hdr;
  int rc;

  if ( (rc=rpm_receive(&rpm_msg_hdr,&rpm_msg_data,RPM_LMAX,NO_BLOCK)) ) return rc;
  rc = rpm_dispatch(&rpm_msg_hdr,&rpm_msg_data);
  if ( rc && rc != GRR_UNKNOWN){
    printf("error return %d from rpm_dispatch for msg of type 0x%x\n",
	   rc,rpm_msg_hdr.type);
    return rc;
  }
  return 0;
}

static int handle_kb(){
  char inp[256];
  int rc = 0;
  if ( !fgets(inp,256,stdin) ) return 1;


  switch(*inp){
  case 'Q': 
  case 'q': 
    run = 0; 
    break;
  case 's':
  case 'S':
    {
      char *line = inp+1;
      int len = strlen(line);
      if (!len) printf("formatted table name required\n");
      else{
	if ( *line == ' ' ) {line++;len--;}
	if ( line[len-1] == '\n' ) len--;
	line[len] = '\0';
	rc = CIF_sub_formatted_table (line,
				      &ftab_updated,
				      ftab_ids + N_ftabs);
	if ( rc ){
	  printf("CIF_sub_formatted_table returned %d for table '%s'\n",rc,line);
	  break;
	}
	else {
	  printf("subscribed to '%s' and got id %d\n",line,ftab_ids[N_ftabs]);
	  N_ftabs++;
	}
      }
    }
    break;
  case '\n':
    break;
  default:
    printf("undefined command %c (0x%x)\n",*inp,*inp);
  case 'h':
  case 'H':
    printf("list of commands:\n");
    printf("\t\033[1mq\033[0m\t\texit program\n");
    printf("\t\033[1mh\033[0m\t\tprint this list\n");
    printf("\t\033[1ms\033[0m ftab\tsubscribe formatted table\n");
  }

  return rc;
}

int main(int argc,const char **argv){
  const char *cna = 0;
  int i;
  int rc;
  rpm_addr mine;
  fd_set all_fds;
  int max_fd;
    
  rpm_init(argc,argv);
  rpm_open(RPM_REGISTER);
  rpm_stat(RPM_MYADDR,sizeof(rpm_addr),&mine);

  for(i=1;i<argc;i++){
    if ( !strncmp(argv[i],"-cna:",5))
      cna = argv[i]+5;
  }

  grr_exec(argv[0]);
  run = 1;

  rc = CIF_msg_init_client (cna,&set_mode,&mine);
  if ( rc ){
    printf("%s:CIF_msg_init_client returned %d\n",argv[0],rc);
    run = 0;
  }

  /* initialization for ITR */
  aframe_store_program_name(argv[0]);

  /* initialize parameters */
  aframe_param_init();
  iol_param_init();
  ITR_leda_param_init();

  /* preferences are loaded here */
  if ( aframe_process_args(argc, 
			   (const char **) argv, AFRAME_CLI) != AFRAME_OK ) {
    aframe_mess("cif_client: main",GRR_SFATAL,"Error processing arguments.");
    return GRR_FISHY;
  }

  itr_client_init(argc, argv);

  FD_ZERO(&all_fds);
  rc = rpi_get_fds(-1,&all_fds); /* wait for all rpi/rpm channels */
  if ( rc ){
    printf("%s:rpi_get_fds returned %d\n",argv[0],rc);
    run = 0;
  }
  FD_SET(STDIN_FILENO, &all_fds);  /* wait for stdin */
  for(i=0;i<128;i++)
    if (FD_ISSET(i,&all_fds)) max_fd = i+1;

  while(run){
    int stat = 0;
    fd_set rfds;
    struct timeval tv;
    
    rfds = all_fds;
    /* Wait up to five seconds. */
    tv.tv_sec = 5;
    tv.tv_usec = 0;
    stat = select(max_fd, &rfds, NULL, NULL, &tv);
    if ( stat<0 )
      printf("select returned %d\n",stat);

    /* try to read from keyboard only if data availlable */
    if ( FD_ISSET(STDIN_FILENO,&rfds) )
      handle_kb();
    /* rpm is save since it is non-blocking */
    handle_rpm();
  }

  rpm_halt();

  return 0;
}

