#include <stdio.h>
#include <stdlib.h>

#include "grr.h"

#include "itr_cif.h"

#include "itrleda.h"
#include "itrleda_structs.h"

#include "ITR_marvin.h"


static int ftabid;

static itrleda_fmtTable_t itr_tables;

int itr_client_init (int argc, const char **argv)
{

    int id;
    /* char *routine = "client_user_init"; */

    printf ("before CIF_sub_formatted_table\n");
    fflush (stdout);
    CIF_sub_formatted_table ("itrfmt:itr_constants", &itrftabhdl, &id);

    ftabid = id;

    /* ---------------------------------------------------
     * Initialisation
     * --------------------------------------------------- */

    ITR_leda_initialize (itrleda_CheckDD_ALWAYS);

    return GRR_OK;
}



void itrftabhdl (const CIF_formatted_tab * tab)
{
    int rc = GRR_OK;
    char *routine = "itrftabhdl";
    itrleda_fmtTable_t *itr_tables_ptr;

    if (tab->id != ftabid) {
	aframe_mess (routine, GRR_SFATAL,
		     "Received tab->id %d, expected ftabid %d\n", tab->id, ftabid);
	return;
    }

    printf ("Received tab->id %d:  ", tab->id);
    itr_tables_ptr = (itrleda_fmtTable_t *) tab->data;

    printf ("formatted at %s\n", itr_tables_ptr->date);
    printf ("   %d entries in chamber table\n", itr_tables_ptr->nChambers);
    printf ("   %d entries in fed table\n", itr_tables_ptr->nFeds);
    printf ("   %d entries in chip table\n", itr_tables_ptr->nChips);
    printf ("   %d entries in chanstat table\n", itr_tables_ptr->nChanstat);
    printf ("   %d entries in pedestal table\n", itr_tables_ptr->nPed);
    printf ("   %d entries in noise table\n", itr_tables_ptr->nNoise);
    printf ("   %d entries in alignment table\n", itr_tables_ptr->nAlign);
    printf ("   %d entries in nesl table\n", itr_tables_ptr->nNesl);
    printf ("   %d entries in nede table\n", itr_tables_ptr->nNede);

    ITR_leda_Chamber_tableFromStruct (itr_tables_ptr->chambers, itr_tables_ptr->nChambers);
    ITR_leda_Fed_tableFromStruct (itr_tables_ptr->fed, itr_tables_ptr->nFeds);
    ITR_leda_Chip_tableFromStruct (itr_tables_ptr->chips, itr_tables_ptr->nChips);
    ITR_leda_Chanstat_tableFromStruct (itr_tables_ptr->chanstat, itr_tables_ptr->nChanstat);
    ITR_leda_Ped_tableFromStruct (itr_tables_ptr->ped, itr_tables_ptr->nPed);
    ITR_leda_Noise_tableFromStruct (itr_tables_ptr->noise, itr_tables_ptr->nNoise);
    ITR_leda_Align_tableFromStruct (itr_tables_ptr->align, itr_tables_ptr->nAlign);
    ITR_leda_nesl_tableFromStruct (itr_tables_ptr->nesl, itr_tables_ptr->nNesl);
    ITR_leda_nede_tableFromStruct (itr_tables_ptr->nede, itr_tables_ptr->nNede);
    ITR_leda_DVOffset_tableFromStruct (itr_tables_ptr->DVOffset, itr_tables_ptr->nOffsets);

    ITR_leda_RelationsFromStruct (itr_tables_ptr);


#if defined(DEBUG_OUTPUT)
    for (chamberIt.begin (); chamberIt.valid (); chamberIt.next ()) {
	LedaTableRow (ITR_Chamber) * chamber = chamberIt.get ();
	LedaRelationIterator (chamber->chip, cha_chip);

	printf ("Chamber 0x%04x :\n", chamber->id);

	for (cha_chip.begin (); cha_chip.valid (); cha_chip.next ()) {
	    LedaTableRow (ITR_Chip) * theChip = cha_chip.get ();
	    LedaTableRow (ITR_Chanstat) * theStatus = LedaRelation (theChip->chanstat);


	    printf ("\t%04x: \t[", theChip->id);

	    if (theStatus) {
		for (cnt = 0; cnt < NCHIPCHAN; cnt++)
		    printf ("%01x", theStatus->status[cnt]);
	    }
	    printf ("]\n");
	}
	printf ("\n");
    }

#endif

/*   rc = itrleda_fetchdata( 0, 0 ); */
/*   if ( rc != GRR_OK ) { */
/*      aframe_mess(routine,GRR_SFATAL, */
/*             "FAILED; marvin NOT initialised  rc = %d",rc); */
/*      return; */
/*   } */

    rc = ITR_silraw_startprogram ();
    if (rc != GRR_OK) {
	aframe_mess (routine, GRR_SFATAL, "FAILED; ITR_silraw NOT initialised  rc = %d", rc);
	return;
    }

    /* initialize marvin */
    ITR_marvin_startrun ();

    return;

}
