/***********************************************************************
 * 
 * itr_ciffmt.c
 *
 * Description:
 *    Routines to format database tables suitable for distribution with
 *    RPM messages.  Used by the L2formatter
 *
 * Use with:
 *    itrfmt.c   this contains main() and the complete initialisation 
 *               for the Inner Tracker Formatter itrfmt
 *
 * Author:   Wolfgang Gradl <gradl@mail.desy.de> 
 *           using the code of the VDS formatter as a template
 *
 * Changes:
 *   
 *
 * $Id: itr_ciffmt.c,v 1.1.1.3 2002/02/07 16:29:20 gradl Exp $
 *
 ***********************************************************************/


#include "itr_cif.h"

#include "itrleda.h"
#include "itrleda_structs.h"

#define N_RTABS 10

typedef struct {
    char *tablename;
    char *tableleda;
    int  major;
    int  minor;
} ITR_table_info_t;

static ITR_table_info_t table_info[N_RTABS] = { 
    {"/ITR_Setup:ITR/Chamber",   "ITR_Chamber", -1, -1 },
    {"/ITR_Setup:ITR/nesl",      "ITR_nesl",    -1, -1 },
    {"/ITR_Setup:ITR/nede",      "ITR_nede",    -1, -1 },
    {"/ITR_Setup:ITR/Fed",       "ITR_Fed",     -1, -1 },
    {"/ITR_Setup:ITR/Chip",      "ITR_Chip",    -1, -1 },
    {"/ITR_MONITOR:ITR/Chanstat","ITR_Chanstat",-1, -1 },
    {"/ITR_MONITOR:ITR/Ped",     "ITR_Ped",     -1, -1 },
    {"/ITR_MONITOR:ITR/Noise",   "ITR_Noise",   -1, -1 },
    {"/ITR_Align:ITR/Align",     "ITR_Align",   -1, -1 },
    {"/ITR_Setup:ITR/DVOffset",  "ITR_DVOffset",-1, -1 }
};


static int ftab_id;
static char *myftab = "itr_constants";
static int itr_rawtabid[N_RTABS];

itrleda_fmtTable_t itr_tables;

int itr_fmtr_subsc (const char *tab, int id, CIF_format_tab ** formatter)
{
    int rc = 0;
    int rtab;

    for (rtab = 0; rtab < N_RTABS; rtab++) {

	rc = CIF_formatter_need_raw (id, table_info[rtab].tablename, 
				     &itr_rawtabid[rtab]);
	if (rc != GRR_OK) {
	    printf ("Error while subscribing to raw table %s: %d (%s)\n",
		    table_info[rtab].tablename, rc, grr_stript (rc));
	    return rc;
	}
    }

    ftab_id = id;
    *formatter = &itr_constant_hdl;

    return rc;
}




int itr_formatter_setup (int argc, const char **argv)
{
    char rpm_itr_const[2000]; /* WARNING:  for RPM to work, strlen(rpm_itr_const) MUST be < 1000 */
    char rpm_preheader[300], rpm_chamber_table[300];
    char rpm_fed_table[300], rpm_chip_table[300];
    char rpm_align_table[300], rpm_chanstat[300];
    char rpm_noise_table[300], rpm_ped_table[300];
    char rpm_nede_table[600], rpm_nesl_table[600];
    char rpm_dvoffset_table[300];
#ifdef FMT_SENDS_IOL_PARAMS
    char rpm_params_table[200];
#endif
    char *routine = "itr_formatter_setup";
    int rc = 0;

    sprintf (rpm_preheader, "major:I,minor:I,date:200B,number:I");

    sprintf (rpm_chamber_table,
	     "nChambers:I,"
	     "Chamber:%d("
	     "id:US,"
	     "name:50B,"
	     "align_flag:B,"
	     "status:B," "type:UB," "chip:6US," "align:US," "tonede:US)", MAX_FMT_CHAMBERS);

    sprintf (rpm_fed_table,
	     "nFeds:I,"
	     "Fed:%d("
	     "id:US,vmeid:UB,board:UB,sharc:UB,link:UB,chip:2US)",
	     MAX_FMT_FEDS);
    

    sprintf (rpm_chip_table,
	     "nChips:I,"
	     "Chip:%d("
	     "id:US,"
	     "nped:UB,"
	     "nnoise:UB," "status:B," "pipelen:UB," "fed:US," "chamber:US)", MAX_FMT_CHIPS);

    sprintf (rpm_chanstat,
	     "nChanstat:I,Chanstat:%d(chipid:US,status:%dUB)", MAX_FMT_CHIPS, NCHIPCHAN);

    sprintf (rpm_ped_table, "nPed:I,Ped:%d(chipid:US,pedestal:%dS)", MAX_FMT_CHIPS, NCHIPCHAN);

    sprintf (rpm_noise_table,
	     "nNoise:I,Noise:%d(chipid:US,rawvar:%dF,subvar:%dF)",
	     MAX_FMT_CHIPS, NCHIPCHAN, NCHIPCHAN);

    sprintf (rpm_align_table,
	     "nAlign:I,"
	     "Align:%d(id:US,nede:I,ang:3F,eang:3F,delt:3F,"
	     "edelt:3F,sloff:3F,esloff:3F,redres:F)", MAX_FMT_ALIGN);

    sprintf (rpm_nesl_table,
	     "nNesl:I,"
	     "Nesl:%d("
	     "inesl:I,nsl:I,mfl:I,lay:I,xf:F,xl:F,yf:F,yl:F,zf:F,zl:F,dx:F,dy:F,"
	     "npl:I,fpl:I,nwl:I,fwl:I,ltyp:I,pnede:I,name:5B,tonede:I)", MAX_FMT_NESL);

    sprintf (rpm_nede_table,
	     "nNede:I,"
	     "Nede:%d("
	     "inede:I,rot:9F,xfw:2F,zfw:F,nwi:2I,xmin:2F,xmax:2F,"
	     "ymin:2F,ymax:2F,zmin:F,zmax:F,sxmn:2F,sxmx:2F,"
	     "symx:2F,symn:2F,acc:F,eff:F,dod:F,pit:F,ms:F,"
	     "nsl:I,superl:I,lay:I,sec:2I,dedx:F,hwc:13B,"
	     "noise:I,ares:I,tonesl:I,tochamber:US,toalign:US)", MAX_FMT_NEDE);

    sprintf (rpm_dvoffset_table,
	     "nOff:I,"
	     "DVOff:%d("
	     "ch:US,off:I)", MAX_FMT_CHAMBERS);
	     	     
#ifdef FMT_SENDS_IOL_PARAMS
    sprintf (rpm_params_table,
	     "nParams:I,"
	     "Params:%d("
	     "value:%dB,module:%dB,verb:%dB",
	     MAX_FMT_PARAMS, PARAM_STRSIZE,PARAM_STRSIZE, PARAM_STRSIZE);
#endif
    
    strcpy (rpm_itr_const, rpm_preheader);
    strcat (rpm_itr_const, ",");
    strcat (rpm_itr_const, rpm_chamber_table);
    strcat (rpm_itr_const, ",");
    strcat (rpm_itr_const, rpm_fed_table); 
    strcat (rpm_itr_const, ","); 
    strcat (rpm_itr_const, rpm_chip_table);
    strcat (rpm_itr_const, ",");
    strcat (rpm_itr_const, rpm_chanstat);
    strcat (rpm_itr_const, ",");
    strcat (rpm_itr_const, rpm_ped_table);
    strcat (rpm_itr_const, ",");
    strcat (rpm_itr_const, rpm_noise_table);
    strcat (rpm_itr_const, ",");
    strcat (rpm_itr_const, rpm_align_table);
    strcat (rpm_itr_const, ",");
    strcat (rpm_itr_const, rpm_nesl_table);
    strcat (rpm_itr_const, ",");
    strcat (rpm_itr_const, rpm_nede_table);
    strcat (rpm_itr_const, ",");
    strcat (rpm_itr_const, rpm_dvoffset_table);

#ifdef FMT_SENDS_IOL_PARAMS
    strcat (rpm_itr_const, ",");
    strcat (rpm_itr_const, rpm_params_table);
#endif

         printf ("RPM Format being used: \n%s\n (len: %d) \n",  
            rpm_itr_const, strlen(rpm_itr_const));                  

    rc = CIF_add_formatted_tab (myftab, rpm_itr_const, &itr_fmtr_subsc);
    if (rc != GRR_OK)
	return rc;

    /* ---------------------------------------------------   
     * Initialization
     * --------------------------------------------------- */

    /* Initilalize some control words */
    aframe_store_program_name (argv[0]);

    /* initialize parameters */
    aframe_param_init ();
    iol_param_init ();
    /* -------------------------------------------------------------
     * start up the Leda stuff 
     * ------------------------------------------------------------ */
    ITR_leda_param_init ();
    ITR_leda_initialize (itrleda_CheckDD_ALWAYS);

    /* ------------------------------------------------------------
     * preferences are loaded here 
     * ------------------------------------------------------------ */
    if (aframe_process_args (argc, (const char **) argv, AFRAME_CLI) != AFRAME_OK) {
	aframe_mess (routine, GRR_SFATAL, "Error processing arguments.");
	return GRR_FISHY;
    }

    /* Name of the Database servers now hardcoded for the itrfmt. WG, 19/oct/2001 */
    /* This is not as bad as it seems, since the server is hardcoded in the 
     * keytable anyway.
     */
    if (strcmp (itrleda_iol.setup_database.value, "/ITR_Setup")) {
	aframe_mess (routine, GRR_SFATAL,
		     "ITR setup database not properly specified! Check iol settings.");
	return GRR_FISHY;
    }

    if (strcmp (itrleda_iol.mon_database.value, "/ITR_MONITOR")) {
	aframe_mess (routine, GRR_SFATAL,
		     "ITR monitor database not properly specified! Check iol settings.");
	return GRR_FISHY;
    }

    if (strcmp (itrleda_iol.align_database.value, "/ITR_Align")) {
	aframe_mess (routine, GRR_SFATAL,
		     "ITR alignment database not properly specified! Check iol settings.");
	return GRR_FISHY;
    }

    /* fetch initial revision of database tables */
    ITR_leda_startprogram (0, 0);

    return rc;
}




int itr_constant_hdl (const CIF_raw_tab * updated_rtabs, int N_raw,
		      CIF_formatted_tab * updated_ftabs, int N_max, int *N_updated)
{

    char *routine = "itr_constant_hdl";
    int j, k, major = 0, minor = 0,
	nCham = 0, nChip = 0, nChanstat = 0, nPed = 0, nNoise = 0, nAlign = 0, 
	nNesl = 0, nNede = 0, nFed = 0, nOffsets = 0,
	rc = GRR_OK;
    extern itrleda_fmtTable_t itr_tables;

    /* -------------------------------------------------------------
     * read in all the DB contents which have been updated 
     * and fill corresponding parts of itr_tables structure 
     * -------------------------------------------------------------- */

    /* Check that the size of the array for the formatted tables is large enough */

    *N_updated = 0;
    if (N_max < 1)
	return GRR_FISHY;

    /* Loop over the updated raw tables, to identify and get them from the DB,
     * and fill their contents in the corresponding parts of the ftab itr_tables */

    printf (">>>>>>>> Number of raw tables:%d\n", N_raw);
    for (j = 0; j < N_raw; j++) {
	printf (">>>>>>>> id:%d, major:%d, minor:%d\n",
		updated_rtabs[j].id, updated_rtabs[j].version[0], updated_rtabs[j].version[1]);
    }

    if (N_raw > 0) {
	for (k = 0; k < N_RTABS; k++) {	/* loop over all rtabs to identify the ID */
	    for (j = 0; j < N_raw; j++) {	/* loop over the updated rtabs */
		if (updated_rtabs[j].id == itr_rawtabid[k]) {
		    printf ("tablename:%s, myid:%d, major:%d, minor:%d\n",
			    table_info[k].tableleda, itr_rawtabid[k],
			    updated_rtabs[j].version[0], updated_rtabs[j].version[1]);

		    table_info[k].major = updated_rtabs[j].version[0];
		    table_info[k].minor = updated_rtabs[j].version[1];
		    
		}
	    }
	}
    }


    /* now fetch the complete set of ITR tables in the correct order */
    for (k = 0; k < N_RTABS; k++) {
	char mess[1024];

	if (table_info[k].major == -1 || table_info[k].minor == -1) {
	    sprintf (mess, "Requesting table %s with no valid [major,minor] number!\n", 
		     table_info[k].tablename);
	    
	    aframe_mess(routine, GRR_SERROR, mess);
	    continue;
	}
	
	sprintf (mess, "Requesting table %s revision [%d,%d]\n", 
		 table_info[k].tablename, table_info[k].major, table_info[k].minor);
	    
	aframe_mess(routine,GRR_SINFO, mess);

	if ((rc = itrleda_fetchtable(table_info[k].tableleda,
				     table_info[k].major, table_info[k].minor)) != GRR_OK) {
	    aframe_mess(routine,GRR_SFATAL,"Failed to open Database or Table.");
	    return GRR_FISHY;
	}
    }

    ITR_leda_Chamber_structFromTable (itr_tables.chambers, MAX_FMT_CHAMBERS, &nCham);
    itr_tables.nChambers = nCham;

    ITR_leda_Fed_structFromTable (itr_tables.fed, MAX_FMT_FEDS, &nFed);
    itr_tables.nFeds = nFed;

    ITR_leda_Chip_structFromTable (itr_tables.chips, MAX_FMT_CHIPS, &nChip);
    itr_tables.nChips = nChip;

    ITR_leda_Chanstat_structFromTable (itr_tables.chanstat, MAX_FMT_CHIPS, &nChanstat);
    if (nChanstat != itr_tables.nChips) {
	printf (" *** warning:  Chanstat table " 
		"and Chip table have different sizes!\n");
    }
    itr_tables.nChanstat = nChanstat;


    ITR_leda_Ped_structFromTable (itr_tables.ped, MAX_FMT_CHIPS, &nPed);
    if (nPed != itr_tables.nChips) {
	printf (" *** warning:  Ped table " 
		"and Chip table have different sizes!\n"); 
    }
    itr_tables.nPed = nPed;

    ITR_leda_Noise_structFromTable (itr_tables.noise, MAX_FMT_CHIPS, &nNoise);
    if (nNoise != itr_tables.nChips) {
	printf (" *** warning:  Noise table " 
		"and Chip table have different sizes!\n");
    }
    itr_tables.nNoise = nNoise;

    ITR_leda_Align_structFromTable (itr_tables.align, MAX_FMT_ALIGN, &nAlign);
    itr_tables.nAlign = nAlign;

    ITR_leda_nesl_structFromTable  (itr_tables.nesl,  MAX_FMT_NESL, &nNesl);
    itr_tables.nNesl  = nNesl;

    ITR_leda_nede_structFromTable  (itr_tables.nede,  MAX_FMT_NEDE, &nNede);
    itr_tables.nNede  = nNede;

    ITR_leda_DVOffset_structFromTable(itr_tables.DVOffset, MAX_FMT_CHAMBERS, &nOffsets);
    itr_tables.nOffsets = nOffsets;

    printf (" ITR/Chamber:  %d entries\n", itr_tables.nChambers);
    printf (" ITR/Chip:     %d entries\n", itr_tables.nChips);
    printf (" ITR/FED:      %d entries\n", itr_tables.nFeds);
    printf (" ITR/Align:    %d entries\n", itr_tables.nAlign);
    printf (" ITR/Nede:     %d entries\n", itr_tables.nNede);
    printf (" ITR/DVOffset: %d entries\n", itr_tables.nOffsets);

    printf (" The number of requested  updates were %d\n", N_raw);
    printf (" The number of fetched updates were %d\n", j);

    strncpy (itr_tables.date, print_date (time (NULL)), 199);

    /* tell CIF where to find the structure with the new formatted table */
    updated_ftabs[0].data = &itr_tables;
    updated_ftabs[0].id   = ftab_id;
    updated_ftabs[0].len  = sizeof itr_tables;
    *N_updated = 1;
    printf ("size of ftab:%d\n", updated_ftabs[0].len);

    return rc;
}



/*
 * Local variables:
 *  c-basic-offset: 4
 * End:
 */
